@@ -13,6 +13,10 @@ class PresstApi(Api):
         self._presst_resource_insts = {}
         self._model_resource_map = {}
 
+    def _init_app(self, app):
+        super(PresstApi, self)._init_app(app)
+        app.presst = self
+
     def get_resource_class(self, reference, module_name=None):
         """
 
@@ -27,7 +31,7 @@ class PresstApi(Api):
 
         :param reference: The resource reference
         :param module_name: module name for lazy loading of class.
-        :returns: :class:`PrestoResource`
+        :return: :class:`PrestoResource`
         """
         if isinstance(reference, PresstResource):  # pragma: no cover
             return reference.__class__
@@ -44,8 +48,7 @@ class PresstApi(Api):
                 else:
                     class_name = reference
                 module = import_module(module_name)
-                # TODO check if this is actually a `Resource`
-                return getattr(module, class_name)
+                return getattr(module, class_name)  # TODO check if this is actually a `Resource`
 
     def parse_resource_uri(self, uri):
         if not uri.startswith(self.prefix):
