@@ -1,12 +1,15 @@
 import datetime
-from flask import request, current_app
+from flask import request, current_app, _request_ctx_stack
 from flask.ext.restful import reqparse, Resource, abort, marshal
 from flask.ext.restful.fields import Boolean, String, Integer, DateTime, Raw
 from flask.ext.sqlalchemy import BaseQuery, Pagination, get_state
 from flask.views import MethodViewType
 from sqlalchemy.dialects import postgres
 from sqlalchemy.orm import class_mapper
-from flask.ext.presst.processor import ProcessorSet
+from flask_presst.processor import ProcessorSet
+from flask_presst.signals import before_create_item, after_create_item, before_create_relationship, \
+    after_create_relationship, before_delete_relationship, after_delete_relationship, before_update_item, \
+    before_delete_item, after_delete_item, after_update_item
 from flask_presst.fields import RelationshipFieldBase, Array, KeyValue, Date
 from flask_presst.nested import NestedProxy
 from flask_presst.parsing import PresstArgument
@@ -170,8 +173,7 @@ class PresstResource(six.with_metaclass(PresstResourceMeta, Resource)):
     def item_get_resource_uri(cls, item):
         if cls.api is None:
             raise RuntimeError("{} has not been registered as an API endpoint.".format(cls.__name__))
-        return '{0}/{1}/{2}'.format(cls.api.prefix, cls.resource_name, six.text_type(
-            getattr(item, cls._id_field, None) or item[cls._id_field])) # FIXME handle both item and attr.
+        return cls.api.url_for(cls, id=getattr(item, cls._id_field, None) or item[cls._id_field])
 
     @classmethod
     def marshal_item(cls, item):
@@ -293,8 +295,7 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
         if isinstance(query, list):
             abort(500, message='Nesting not supported for this resource.')
 
-        cls._processors.filter(request.method, query, cls)
-        return query
+        return cls._processors.filter(query, request.method, cls)
 
     @classmethod
     def get_item_list_for_relationship(cls, relationship, parent_item):
@@ -303,14 +304,16 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
         if isinstance(query, list):
             abort(500, message='Nesting not supported for this resource.')
 
-        cls._processors.filter(request.method, query, cls)
-        return query
+        return cls._processors.filter(query, request.method, cls)
 
     @classmethod
     def create_item_relationship(cls, id_, relationship, parent_item):
         item = cls.get_item_for_id(id_)
 
-        cls._processors.before_create_relationship(parent_item, relationship, item)
+        before_create_relationship.send(cls,
+                                        parent_item=parent_item,
+                                        relationship=relationship,
+                                        item=item)
 
         session = cls._get_session()
 
@@ -321,14 +324,20 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
             session.rollback()
             raise
 
-        cls._processors.after_create_relationship(parent_item, relationship, item)
+        after_create_relationship.send(cls,
+                                       parent_item=parent_item,
+                                       relationship=relationship,
+                                       item=item)
         return item
 
     @classmethod
     def delete_item_relationship(cls, id_, relationship, parent_item):
         item = cls.get_item_for_id(id_)
 
-        cls._processors.before_delete_relationship(parent_item, relationship, item)
+        before_delete_relationship.send(cls,
+                                        parent_item=parent_item,
+                                        relationship=relationship,
+                                        item=item)
 
         session = cls._get_session()
 
@@ -339,7 +348,10 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
             session.rollback()
             raise
 
-        cls._processors.after_delete_relationship(parent_item, relationship, item)
+        after_delete_relationship.send(cls,
+                                       parent_item=parent_item,
+                                       relationship=relationship,
+                                       item=item)
 
     @classmethod
     def get_item_for_id(cls, id_):
@@ -352,7 +364,7 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
         for key, value in six.iteritems(dct):
             setattr(item, key, value)
 
-        cls._processors.before_create_item(item, cls)
+        before_create_item.send(cls, item=item)
 
         session = cls._get_session()
 
@@ -363,7 +375,7 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
             session.rollback()
             raise
 
-        cls._processors.after_create_item(item, cls)
+        after_create_item.send(cls, item=item)
         return item
 
     @classmethod
@@ -373,7 +385,7 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
         session = cls._get_session()
 
         try:
-            cls._processors.before_update_item(item, dct, partial, cls)
+            before_update_item.send(cls, item=item, changes=dct, partial=partial)
 
             for key, value in six.iteritems(dct):
                 setattr(item, key, value)
@@ -383,20 +395,20 @@ class ModelResource(six.with_metaclass(ModelResourceMeta, PresstResource)):
             session.rollback()
             raise
 
-        cls._processors.after_update_item(item, cls)
+        after_update_item.send(cls, item=item)
         return item
 
     @classmethod
     def delete_item(cls, id_):
         item = cls.get_item_for_id(id_)
 
-        cls._processors.before_delete_item(item, cls)
+        before_delete_item.send(cls, item=item)
 
         session = cls._get_session()
         session.delete(item)
         session.commit()
 
-        cls._processors.after_delete_item(item, cls)
+        after_delete_item.send(cls, item=item)
 
     _pagination_parser = reqparse.RequestParser()
     _pagination_parser.add_argument('per_page', location='args', type=int, default=20) # 20
